package edu.nyu.scps.zip;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import android.app.ListActivity;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteException;
import android.database.sqlite.SQLiteOpenHelper;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.LinearLayout;
import android.widget.ListAdapter;
import android.widget.ListView;
import android.widget.SimpleCursorAdapter;
import android.widget.TextView;
import android.widget.Toast;

public class ZipActivity extends ListActivity {
	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		String[] fields = new String[] {
			"zipcode",
			"city",
			"state",
			"latitude",
			"longitude",
			//"county"
		};

		int[] textViews = new int[] {
			R.id.zipcode,
			R.id.city,
			R.id.state,
			R.id.latitude,
			R.id.longitude,
			//R.id.county
		};

		ListAdapter adapter = new SimpleCursorAdapter(
			this,
			R.layout.list_item,
			new Helper(this).getCursor(),
			fields,
			textViews
		);
		
		setListAdapter(adapter);
		ListView listView = getListView();
		listView.setTextFilterEnabled(true);	//Type characters to select an item.

		listView.setOnItemClickListener(new OnItemClickListener() {
			@Override
			public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
				TextView city = (TextView)((LinearLayout)view).getChildAt(1);
				String s = String.format("\"%s\" position == %d, id == %d", 
						city.getText(), position, id);
				Toast.makeText(ZipActivity.this, s, Toast.LENGTH_LONG).show();  
			}
		});
	}

	public class Helper extends SQLiteOpenHelper {
		private static final String DB_PATH = "/data/data/edu.nyu.scps.zip/databases/";
		private static final String DB_NAME = "zipcode.sqlite";
		Context context;

		public Helper(Context context) {
			super(context, DB_PATH + DB_NAME, null, 1);
			this.context = context;
		}

		@Override
		public void onCreate(SQLiteDatabase db) {
		}

		@Override
		public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
		}

		public Cursor getCursor() {

			if (!dataBaseExists()) {
				try {
					createDataBase();
				} catch (IOException exception) {
					Toast.makeText(context, exception.toString(), Toast.LENGTH_LONG).show();
				}
			}

			SQLiteDatabase database = getReadableDatabase();	//call onCreate

			//Prepend enough zeroes to zipcode to make it a five-digit number.
			String select = "SELECT _id, substr('00000' || zipcode, -5, 5) AS zipcode, "
				+ "city, state, latitude, longitude, county FROM zipcodes;";

			Cursor cursor = database.rawQuery(select, null);
			cursor.moveToFirst();
			return cursor;
		}
		
		//Return true if database file already exists.

		private boolean dataBaseExists() {
			SQLiteDatabase dataBase = null;

			try {
				dataBase = SQLiteDatabase.openDatabase(DB_PATH + DB_NAME, null, SQLiteDatabase.OPEN_READONLY);
			} catch (SQLiteException exception) {
				//database doesn't exist yet.
			}

			if (dataBase != null) {
				dataBase.close();
			}

			return dataBase != null;
		}

		private void createDataBase() throws IOException {
			InputStream inputStream = context.getAssets().open(DB_NAME); //in assets folder
			OutputStream outputStream = new FileOutputStream(DB_PATH + DB_NAME);

			byte[] buffer = new byte[1024];
			int length;

			while ((length = inputStream.read(buffer)) > 0) {
				outputStream.write(buffer, 0, length);
			}

			outputStream.flush();
			outputStream.close();
			inputStream.close();
		}
	}
}